@@ -1,41 +1,73 @@
 context('popkin_Rdata')
 
-# message('getwd: ', getwd())
-
-# # introduce an error on purpose
-# test_that("testing is run at all!", {
-#     expect_equal(1, 2)
-# })
+# used in some examples
+library(ape)
 
 # loads Rdata matrices to test
 load('Xs.RData')
 
 # start with lower-level/internal tests, more informative that higher-level function errors
 
+test_that( "M is correct", {
+    # the M used in other tests was pre-calculated by popkin_A
+    # here we calculate it from X the way we do it elsewhere (including popkin_af), make sure it agrees
+
+    # popkin removes loci that are fixed (not in M count)
+    x_hat <- rowMeans( X, na.rm = TRUE )
+    # indexes to keep
+    indexes <- !is.na( x_hat ) & x_hat > 0 & x_hat < 2
+    # filter loci
+    X2 <- X[ indexes, ]
+
+    # now calculate M
+    M_direct <- crossprod( !is.na( X2 ) )
+    # in this case M_direct gets labels from X, let's remove those (M doesn't have them)
+    dimnames( M_direct ) <- NULL
+    # matching requires labels to agree
+    expect_equal( M, M_direct )
+})
+
 test_that("validate_kinship works", {
+    # try default "error" versions alongside logical versions
+    
     # validate positive examples
     expect_silent( validate_kinship( Phi ) )
     expect_silent( validate_kinship( Phi0 ) )
     expect_silent( validate_kinship( A, name = 'A' ) ) # not real kinship but satisfies requirements
+    expect_true( validate_kinship( Phi, logical = TRUE ) )
+    expect_true( validate_kinship( Phi0, logical = TRUE ) )
+    expect_true( validate_kinship( A, name = 'A', logical = TRUE ) ) # not real kinship but satisfies requirements
 
     # negative examples
     # dies if input is missing
     expect_error( validate_kinship() )
+    expect_error( validate_kinship( logical = TRUE ) ) # still dies here
+    
+    # NULL values should fail (important for `plot_popkin`)
+    expect_error( validate_kinship( NULL ) )
+    expect_false( validate_kinship( NULL, logical = TRUE ) )
     
     # and if input is not a matrix
     expect_error( validate_kinship( 1:5 ) )
+    expect_false( validate_kinship( 1:5, logical = TRUE ) )
     
     # and for non-numeric matrices
     char_mat <- matrix(c('a', 'b', 'c', 'd'), nrow=2)
     expect_error( validate_kinship( char_mat ) )
+    expect_false( validate_kinship( char_mat, logical = TRUE ) )
     
     # and non-square matrices
     non_kinship <- matrix(1:2, nrow=2)
     expect_error( validate_kinship( non_kinship ) )
+    expect_false( validate_kinship( non_kinship, logical = TRUE ) )
     
     # and non-symmetric matrices
+    # (also test `sym = FALSE` option that lets this case pass)
     non_kinship <- matrix(1:4, nrow=2)
     expect_error( validate_kinship( non_kinship ) )
+    expect_silent( validate_kinship( non_kinship, sym = FALSE ) )
+    expect_false( validate_kinship( non_kinship, logical = TRUE ) )
+    expect_true( validate_kinship( non_kinship, sym = FALSE, logical = TRUE ) )
 })
 
 
@@ -131,6 +163,7 @@ test_that("solve_m_mem_lim works", {
 })
 
 test_that("function returns precomputed values: weights_subpops", {
+    # these are extremely trivial (sad choice but meh)
     expect_equal(weights_subpops(subpops0), w0)
     expect_equal(weights_subpops(subpops), w)
     # make sure dimensions match
@@ -145,6 +178,73 @@ test_that("function returns precomputed values: weights_subpops", {
     expect_true(all(w < 1))
 })
 
+test_that("weights_subpops works on random data", {
+    # cause errors on purpose
+    # missing arguments, etc
+    expect_error( weights_subpops() )
+    expect_error( weights_subpops( NULL ) )
+    expect_error( weights_subpops( c('a', NA ) ) ) # no NAs
+    
+    # construct a large but random `subpops` vector
+    # (this is the real test of correctness)
+    n <- 300
+    subpops <- sample( letters, n, replace = TRUE )
+    expect_silent(
+        w <- weights_subpops( subpops )
+    )
+    # make sure dimensions match
+    expect_equal( length( w ), n )
+    # test the basic qualities of weights
+    expect_equal( sum( w ), 1 )
+    expect_true( all( w > 0 ) )
+    expect_true( all( w < 1 ) )
+    # can check that every subpop has equal weight too
+    K <- length( unique( subpops ) )
+    w_by_subpops_obs <- aggregate( w, list( subpops = subpops ), sum )$x
+    w_by_subpops_exp <- rep.int( 1 / K, K )
+    expect_equal( w_by_subpops_obs, w_by_subpops_exp )
+
+    # construct 2-level hierarchy case
+    # have top level have fewer choices to minimize the chance of empty and trivial cases
+    subpops <- sample( letters[1:3], n, replace = TRUE )
+    #subsubpops <- sample( letters[1:3], n, replace = TRUE )
+    subsubpops <- sample( letters, n, replace = TRUE )
+    # to ensure nestedness, sub-subpopulations get unique suffixes by subpop
+    subsubpops <- paste0( subsubpops, subpops )
+    expect_silent(
+        w <- weights_subpops( subpops, subsubpops )
+    )
+    # make sure dimensions match
+    expect_equal( length( w ), n )
+    # test the basic qualities of weights
+    expect_equal( sum( w ), 1 )
+    expect_true( all( w > 0 ) )
+    expect_true( all( w < 1 ) )
+    # can check that every subpop has equal weight too
+    K <- length( unique( subpops ) )
+    w_by_subpops_obs <- aggregate( w, list( subpops = subpops ), sum )$x
+    w_by_subpops_exp <- rep.int( 1 / K, K )
+    expect_equal( w_by_subpops_obs, w_by_subpops_exp )
+    # and for this case, each sub-subpopulation has equal weight within its subpopulation
+    w_by_subsubpops_obs <- aggregate( w, list( subsubpops = subsubpops ), sum )$x
+    # expectation here is harder to construct, but we do it!
+    # this should result in same weights per sub-subpopulation, essentially converting them to individuals
+    tab <- unique( data.frame( subpops = subpops, subsubpops = subsubpops ) )
+    # have to sort subsubpops to be in same order as aggregate returns them
+    tab <- tab[ order( tab$subsubpops ), ]
+    labs2 <- tab$subpops
+    w_by_subsubpops_exp <- weights_subpops( labs2 )
+    expect_equal( w_by_subsubpops_obs, w_by_subsubpops_exp )
+
+    # cause an error on purpose for non-nested subpopulations
+    subpops <- c(1, 1, 2, 2)
+    subsubpops <- c(1, 2, 1, 2)
+    expect_error( weights_subpops( subpops, subsubpops ) )
+    # and lenghts that don't match
+    subsubpops <- 1:5
+    expect_error( weights_subpops( subpops, subsubpops ) )
+})
+
 test_that("function returns precomputed values: popkin_A", {
     # standard test (X is integer)
     expect_silent( obj <- popkin_A( X ) )
@@ -420,6 +520,36 @@ test_that("n_eff works", {
     expect_true(min(obj$weights) < 0) # this example must have negative weights, or it's useless!
 })
 
+test_that("phylo_max_edge works", {
+    # plot examples with trees
+    tree <- rtree( 3 )
+
+    expect_silent(
+        max_length <- phylo_max_edge( tree )
+    )
+    expect_true( is.numeric( max_length ) )
+    expect_equal( length( max_length ), 1 )
+    expect_true( max_length >= 0 )
+    expect_true( max_length <= sum( tree$edge.length ) )
+})
+
+test_that("plot_phylo works", {
+    # just in case some standalone defaults don't work as they should
+    
+    # set up a temporary path to write to
+    fo <- tempfile('test-plot-phylo', fileext = '.pdf')
+    # random sample tree
+    tree <- rtree( 3 )
+    
+    # this should work
+    pdf( fo, width = 14 )
+    par(oma = c(0, 1.5, 0, 3))
+    par(mar = c(0, 0, 2, 0) + 0.2)
+    expect_silent( plot_phylo( tree ) )
+    invisible( dev.off() )
+    invisible( file.remove(fo) )
+})
+
 test_that("plot_popkin works", {
     # set up a temporary path to write to
     fo <- tempfile('test-plot-popkin', fileext = '.pdf')
@@ -481,4 +611,233 @@ test_that("plot_popkin works", {
     expect_silent( plot_popkin( inbr_diag( list(Phi, NULL) ), titles = 'a', null_panel_data = FALSE ) )
     invisible( dev.off() )
     invisible( file.remove(fo) )
+
+    # plot examples with trees
+    tree <- rtree( 3 )
+
+    # this should work
+    pdf( fo, width = 14 )
+    par(oma = c(0, 1.5, 0, 3))
+    par(mar = c(0, 0, 2, 0) + 0.2)
+    expect_silent( plot_popkin( list( tree, inbr_diag( Phi ), NULL) ) )
+    invisible( dev.off() )
+    invisible( file.remove(fo) )
+    
+})
+
+#################
+### popkin_af ###
+#################
+
+# for the following tests, simulate a matrix with subpopulation allele frequencies
+n <- 10
+m <- 100 # P's is smaller than X's m
+# group subpopulations in pairs, just for this test
+subpops <- ceiling( (1 : n) / 2 )
+# unstructured allele frequencies, meh
+P <- matrix(
+    runif( n * m ),
+    nrow = m,
+    ncol = n
+)
+# add labels for tests
+colnames( P ) <- paste0( 'S', 1 : n )
+rownames( P ) <- paste0( 'L', 1 : m )
+# true M when there's no missingness
+M_true <- matrix( m, n, n )
+colnames( M_true ) <- colnames( P )
+rownames( M_true ) <- colnames( P )
+
+# make a copy of P with random missingness
+miss <- 0.1 # missingness rate
+P_miss <- P
+P_miss[ sample( n * m, n * m * miss ) ] <- NA
+# NOTE: if any rows are entirely fixed or missing, popkin_af should still work!
+# this is true M for missingness case
+M_miss_true <- crossprod( !is.na( P_miss ) )
+
+test_that( "true M are ok", {
+    # M matrix didn't have equivalent in past runs, test in more detail here
+    expect_true( is.matrix( M_true ) )
+    expect_equal( nrow( M_true ), n )
+    expect_equal( ncol( M_true ), n )
+    expect_true( all( M_true == m ) )
+    
+    # repeat with missingness
+    expect_true( is.matrix( M_miss_true ) )
+    expect_equal( nrow( M_miss_true ), n )
+    expect_equal( ncol( M_miss_true ), n )
+    # with missingness we just have bounds
+    expect_true( all( M_miss_true <= m ) )
+    expect_true( all( M_miss_true >= 0 ) )
+})
+
+test_that( "popkin_af_basic works", {
+    # expect error if P is missing
+    expect_error( popkin_af_basic( ) )
+    # cause error if subpopulations and P don't match in dimensions
+    expect_error( popkin_af_basic( P, subpops = subpops[ -1 ] ) )
+
+    # now proper run
+    # NOTE: saving `coancestry` as global so next block can compare against it
+    expect_silent(
+        coancestry <<- popkin_af_basic( P )
+    )
+    # check dimensions and data ranges
+    expect_true( is.numeric( coancestry ) )
+    expect_true( is.matrix( coancestry ) )
+    expect_equal( nrow( coancestry ), n )
+    expect_equal( ncol( coancestry ), n )
+    expect_true( !anyNA( coancestry ) )
+    # because of the way Amin is picked here (subpops = NULL), the minimum is exactly zero
+    expect_true( all( coancestry >= 0 ) )
+    expect_true( all( coancestry <= 1 ) )
+    # test label transfer
+    expect_equal( colnames( coancestry ), colnames( P ) )
+    expect_equal( rownames( coancestry ), colnames( P ) )
+
+    # test want_M version
+    expect_silent(
+        obj <- popkin_af_basic( P, want_M = TRUE )
+    )
+    # require equality here with previous run
+    expect_equal( obj$coancestry, coancestry )
+    # this is not a matrix in this case, but just to prepare for version with missingness later
+    expect_equal( obj$M, m )
+    
+    # run transposed version
+    coancestry2 <- popkin_af_basic( t( P ), loci_on_cols = TRUE )
+    # require equality here with previous run
+    expect_equal( coancestry2, coancestry )
+    
+    # run with subpops
+    expect_silent(
+        coancestry <- popkin_af_basic( P, subpops = subpops )
+    )
+    # check dimensions and data ranges
+    expect_true( is.numeric( coancestry ) )
+    expect_true( is.matrix( coancestry ) )
+    expect_equal( nrow( coancestry ), n )
+    expect_equal( ncol( coancestry ), n )
+    expect_true( !anyNA( coancestry ) )
+    # because of the way Amin is picked here (paired subpops), the minimum is slightly below zero
+    expect_true( all( coancestry >= -1 ) )
+    expect_true( all( coancestry <= 1 ) )
+    # test label transfer
+    expect_equal( colnames( coancestry ), colnames( P ) )
+    expect_equal( rownames( coancestry ), colnames( P ) )
+})
+
+test_that( "popkin_af_basic_na works", {
+    # expect error if P is missing
+    expect_error( popkin_af_basic_na( ) )
+    # cause error if subpopulations and P don't match in dimensions
+    expect_error( popkin_af_basic_na( P, subpops = subpops[ -1 ] ) )
+
+    # now proper run
+    # here we use P without missingness, expect to match previous version
+    expect_silent(
+        coancestry2 <- popkin_af_basic_na( P )
+    )
+    expect_equal( coancestry2, coancestry )
+
+    # and want_M version
+    expect_silent(
+        obj <- popkin_af_basic_na( P, want_M = TRUE )
+    )
+    expect_equal( obj$coancestry, coancestry )
+    expect_equal( obj$M, M_true )
+
+    # now run with P with missingness!
+    # export this one too
+    expect_silent(
+        coancestry_miss <<- popkin_af_basic_na( P_miss )
+    )
+    # check dimensions and data ranges
+    expect_true( is.numeric( coancestry_miss ) )
+    expect_true( is.matrix( coancestry_miss ) )
+    expect_equal( nrow( coancestry_miss ), n )
+    expect_equal( ncol( coancestry_miss ), n )
+    expect_true( !anyNA( coancestry_miss ) )
+    # because of the way Amin is picked here (subpops = NULL), the minimum is exactly zero
+    expect_true( all( coancestry_miss >= 0 ) )
+    expect_true( all( coancestry_miss <= 1 ) )
+    # test label transfer
+    expect_equal( colnames( coancestry_miss ), colnames( P ) )
+    expect_equal( rownames( coancestry_miss ), colnames( P ) )
+
+    # want_M version with missigness
+    expect_silent(
+        obj_miss <- popkin_af_basic_na( P_miss, want_M = TRUE )
+    )
+    # compare to proper calculation from data with missingness
+    expect_equal( obj_miss$coancestry, coancestry_miss )
+    expect_equal( obj_miss$M, M_miss_true )
+    
+    # and transposed version (stick with want_M to test more things)
+    expect_silent(
+        obj_miss2 <- popkin_af_basic_na( t( P_miss ), want_M = TRUE, loci_on_cols = TRUE )
+    )
+    # test whole object at once
+    expect_equal( obj_miss2, obj_miss )
+
+    # run with subpops
+    expect_silent(
+        obj_miss_subpops <- popkin_af_basic_na( P_miss, want_M = TRUE, subpops = subpops )
+    )
+    # extract elements
+    coancestry_miss_subpops <- obj_miss_subpops$coancestry
+    # check dimensions and data ranges
+    expect_true( is.numeric( coancestry_miss_subpops ) )
+    expect_true( is.matrix( coancestry_miss_subpops ) )
+    expect_equal( nrow( coancestry_miss_subpops ), n )
+    expect_equal( ncol( coancestry_miss_subpops ), n )
+    expect_true( !anyNA( coancestry_miss_subpops ) )
+    # because of the way Amin is picked here (paired subpops), the minimum is slightly below zero
+    expect_true( all( coancestry_miss_subpops >= -1 ) )
+    expect_true( all( coancestry_miss_subpops <= 1 ) )
+    # test label transfer
+    expect_equal( colnames( coancestry_miss_subpops ), colnames( P ) )
+    expect_equal( rownames( coancestry_miss_subpops ), colnames( P ) )
+    # M is the same here whether subpops are used or not
+    expect_equal( obj_miss_subpops$M, obj_miss$M )
+})
+
+# this tests final, exported version (the other ones are for checks only)
+test_that( "popkin_af works", {
+    # expect error if P is missing
+    expect_error( popkin_af( ) )
+    # cause error if subpopulations and P don't match in dimensions
+    expect_error( popkin_af( P, subpops = subpops[ -1 ] ) )
+
+    # now proper run
+    # here we use P without missingness, expect to match previous version
+    expect_silent(
+        coancestry2 <- popkin_af( P )
+    )
+    expect_equal( coancestry2, coancestry )
+    # repeat with want_M
+    expect_silent(
+        obj <- popkin_af( P, want_M = TRUE )
+    )
+    expect_equal( obj$coancestry, coancestry )
+    expect_equal( obj$M, M_true )
+
+    # repeat with P with missingness
+    expect_silent(
+        coancestry2 <- popkin_af( P_miss )
+    )
+    expect_equal( coancestry2, coancestry_miss )
+    # repeat with want_M
+    expect_silent(
+        obj <- popkin_af( P_miss, want_M = TRUE )
+    )
+    expect_equal( obj$coancestry, coancestry_miss )
+    expect_equal( obj$M, M_miss_true )
+
+    # test transposed version
+    expect_silent(
+        obj2 <- popkin_af( t( P_miss ), want_M = TRUE, loci_on_cols = TRUE )
+    )
+    expect_equal( obj2, obj )
 })
